-- Variables
local EMISSARY_COUNT = 3;

-- Settings
local settingsFrame = CreateFrame("Frame", "DLUSettingsFrame");

-- Notification System
CreateFrame("Frame", "DLUNotificationFrame");
DLUNotificationFrame:SetAlpha(0);
DLUNotificationFrame:SetWidth(320);
DLUNotificationFrame:SetHeight(80);
DLUNotificationFrame:SetPoint("TOP", UIParent, "TOP", 0, 0);
DLUNotificationFrame:CreateFontString("DLUNotification", "ARTWORK", "GameFontNormalLarge");
DLUNotification:SetPoint("CENTER", 0, 0);

-- Change Character UI
hooksecurefunc("PaperDollFrame_UpdateStats", function()
	DLU.showStats("player");
end)

-- Global UI functions
function DLU.getEmissaryValue(id)
	local currentEmissary = DLU.emissaryTable[id];
	local value;
	
	if (currentEmissary == nil) then
		value = "Already completed";
	elseif (currentEmissary.completed == currentEmissary.required) then
		value = "Ready to turn-in";
	elseif (DLU.player.level < 110) then
		value = "Not max level yet...";
	else
		value = format("%i/%i", currentEmissary.completed, currentEmissary.required);
	end
	
	return value;
end

function DLU.updateEmissaries()
	DLU.emissaryTable = DLU.getEmissaries();
	local amount = #DLU.emissaryTable;
	
	DLU.resetEmissaryUi();
	
	for i = 1, amount do
		local value = DLU.getEmissaryValue(i);
		
		if (WQCompanion.EmissariesContainer.Emissaries[i].Label:GetText() == nil) then
			if (DLU.emissaryTable[i] ~= nil) then
				local currentEmissary = DLU.emissaryTable[i];
				
				if (WQCompanion.EmissariesContainer.Emissaries[i].questId == 0) then
					WQCompanion.EmissariesContainer.Emissaries[i].questId = currentEmissary.questId;
				end
				
				WQCompanion.EmissariesContainer.Emissaries[i].Label:SetText(DLU.iconToText(currentEmissary.icon));
				WQCompanion.EmissariesContainer.Emissaries[i].Tooltip = format("%s", currentEmissary.factionName);
			else
				WQCompanion.EmissariesContainer.Emissaries[i].Label:SetText("");
				WQCompanion.EmissariesContainer.Emissaries[i].Tooltip = format("Already completed");
			end
		end
		
		WQCompanion.EmissariesContainer.Emissaries[i].Value:SetText(value);
	end
	
	--[[
	if (DLU.player.level >= 110) then
		local left = 3 - amount;
		
		for i = 3, left do
			local currentEmissary = DLU.emissaryTable[i];
			if (currentEmissary == nil) then
				local value = DLU.getEmissaryValue(i);
				WQCompanion.EmissariesContainer.Emissaries[i].Label:SetText("");
				WQCompanion.EmissariesContainer.Emissaries[i].Value:SetText(value)
				WQCompanion.EmissariesContainer.Emissaries[i].Tooltip = "";
			end
		end
	end
	]]
end

function DLU.resetEmissaryUi()
	for i = 1, EMISSARY_COUNT do
		WQCompanion.EmissariesContainer.Emissaries[i].Label:SetText("");
		WQCompanion.EmissariesContainer.Emissaries[i].Value:SetText("Already completed");
		WQCompanion.EmissariesContainer.Emissaries[i].Tooltip = "Already completed";
		WQCompanion.EmissariesContainer.Emissaries[i].questId = 0;
	end
end

function DLU.getQuestTimeLeft(questId)
	local timeLeftMinutes = C_TaskQuest.GetQuestTimeLeftMinutes(questId);
	local timeString = format("Timeleft: %i", timeLeftMinutes);
	
	if (timeLeftMinutes ~= nil) then
		if (timeLeftMinutes < 60) then
			timeString = format("Minutes left: %i", SecondsToTime(timeLeftMinutes * 60));
		elseif (timeLeftMinutes < 24 * 60) then
			timeString = format("Hours left: %i", floor(timeLeftMinutes / 60));
		else
			timeString = format("Days left: %i", floor(timeLeftMinutes) / 1440);
		end
	end
	
	return timeString;
end

-- UI related functions
local function achievementCompleted(achievementId)
	local _, _, _, _, _, _, _, _, _, _, _, _, wasEarnedByMe = GetAchievementInfo(achievementId);
	return wasEarnedByMe;
end

local function addToTracker(achievementId, button)
	if (achievementCompleted(achievementId)) then
		RemoveTrackedAchievement(achievementId);
		if (button ~= nil) then
			disableButton(button, "Already completed!");
		end
	else
		AddTrackedAchievement(achievementId);
	end
end

local function removeFromTracker(achievementId)
	local isTracked = IsTrackedAchievement(achievementId);
	if (isTracked) then
		RemoveTrackedAchievement(achievementId);
	end
end

local function OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR");
	GameTooltip:SetText(self.Tooltip);
	
	if (self.questId > 0) then
		local timeString = DLU.getQuestTimeLeft(self.questId);
		GameTooltip:AddLine(format("%s", timeString));
	end
	
	local selfValue = self.Value:GetText();
	local completed, required = selfValue:match("(%d+)/(%d+)");
	
	if (completed and required) then
		GameTooltip:AddLine(format("Completed: %i/%i", completed, required), 1, 1, 1);
	else
		GameTooltip:AddLine(self.Value:GetText(), 1, 1, 1);
	end
	
	GameTooltip:Show();
end

local function OnLeave(self)
	GameTooltip:Hide()
end

-- Shortage
function DLU.checkShortage(dID, classRolesAvailable)
	for i = 1, LFG_ROLE_NUM_SHORTAGE_TYPES do
		local eligible, tank, healer, damage, itemCount = GetLFGRoleShortageRewards(dID, i);
		local tankLocked, healerLocked, damageLocked = GetLFDRoleRestrictions(dID);
		tank = tank and classRolesAvailable.tank and not tankLocked;
		healer = healer and classRolesAvailable.heal and not healerLocked;
		damage = damage and classRolesAvailable.dps and not damageLocked;
		if(eligible and itemCount > 0 and (tank or healer or damage)) then
			local data = {dID = dID, tank = tank, heal = healer, dps = damage};
			return data;
		end
	end
	
	return nil;
end

function DLU.createShortageList()
	DLU.shortageList = {};
	
	local canTank, canHealer, canDamage = C_LFGList.GetAvailableRoles()
	local classRolesAvailable = {tank = canTank, heal = canHealer, dps = canDamage};
	
	for i = 1, GetNumRFDungeons() do
		local dID = GetRFDungeonInfo(i);
		local shortage = DLU.checkShortage(dID, classRolesAvailable);
		table.insert(DLU.shortageList, shortage);
	end
	
	for i = 1, GetNumRandomDungeons() do
        local dID = GetLFGRandomDungeonInfo(i)
		local shortage = DLU.checkShortage(dID, classRolesAvailable);
		table.insert(DLU.shortageList, shortage);
    end
	
	for i = 1, #WQCompanion.ERW.Content.lines do
		WQCompanion.ERW.Content.lines[i].text:SetText("");
	end
end

-- UI functions
local uniquealyzer = 1;
local checkButtons = {};

local function createCheckbutton(parent, x_loc, y_loc, displayname, option)
	uniquealyzer = uniquealyzer + 1;
	
	local checkbutton = CreateFrame("CheckButton", "checkbutton_0" .. uniquealyzer, parent, "ChatConfigCheckButtonTemplate");
	checkbutton:ClearAllPoints()
	checkbutton:SetPoint("TOPLEFT", x_loc, y_loc);
	getglobal(checkbutton:GetName() .. 'Text'):SetText(displayname);
	
	-- Option checkbox
	if (option) then
		checkButtons[option] = checkbutton;
	end

	return checkbutton;
end

function DLU.createTextRoot(text, parent, x_loc, y_loc, fontType)
	local title = parent:CreateFontString(nil, "ARTWORK", fontType);
	title:SetPoint("TOPLEFT", x_loc, y_loc);
	title:SetText(text);
	return title;
end

function DLU.createTextChild(text, parent, below, x_loc, y_loc, fontType)
	local textChildNode = parent:CreateFontString(nil, "ARTWORK", fontType);
	textChildNode:SetPoint("TOPLEFT", below, "BOTTOMLEFT", x_loc, y_loc);
	textChildNode:SetText(text);
	return textChildNode;
end

local function createButton(parent, text, width, height, tooltip, x_loc, y_loc)
	local button = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate");
	button:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", x_loc, y_loc);
    button:SetText(text);
    button.tooltipText = tooltip;
    button:SetWidth(width);
    button:SetHeight(height);
	return button;
end

local function disableButton(button, text)
	button:SetText(text);
	button:Disable();
end

local function SetUpAddonOptions()
	local panel = CreateFrame("Frame", "DLUOptionsPanel", InterfaceOptionsFramePanelContainer);
	-- Option panel name
	panel.name = DLU.addonName;
	-- Add info to panel
	local title = DLU.createTextRoot(DLU.addonName, panel, 16, -16, "GameFontNormalLarge");
	local authorText = DLU.createTextChild("Made by: " .. GetAddOnMetadata(DLU.addonName, "Author"), panel, title, 0, -8, "GameFontHighlightSmall");
	local versionText = DLU.createTextChild("Version: ".. DLU.addonVersion, panel, authorText, 0, -8, "GameFontHighlightSmall");
	local descriptionText = DLU.createTextChild("Description: " .. GetAddOnMetadata(DLU.addonName, "X-Notes"), panel, versionText, 0, -8, "GameFontHighlightSmall");
	local websiteText = DLU.createTextChild("Author website: " .. GetAddOnMetadata(DLU.addonName, "X-Website") .. " (it's on Danish)", panel, descriptionText, 0, -8, "GameFontHighlightSmall");
	-- Add help button
	local helpButton = createButton(panel, "Help", 80, 22, "Shows a list of commands, that you can use.", 10, 30);
	helpButton:SetScript("OnClick", function()
		SlashCmdList.DLU("help");
	end);
	-- Add WQ Companion button'
	local wqButton = createButton(panel, "Open WQ Companion", 160, 22, "Opens up the WQ Companion UI.", 100, 30);
	wqButton:SetScript("OnClick", function()
		SlashCmdList.DLU("wqc");
	end);
	-- Add panel to addon options
	InterfaceOptions_AddCategory(panel);
	
	-- SettingsPanel
	local settingsPanel = CreateFrame("Frame", "DLUSettings", UIParent);
	settingsPanel.name = "Settings";
	settingsPanel.parent = panel.name;
	InterfaceOptions_AddCategory(settingsPanel);
	-- SettingsPanel text
	local settingsPanelTitle = DLU.createTextRoot(settingsPanel.name, settingsPanel, 16, -16, "GameFontNormalLarge");
	local settingsPanelDescription = DLU.createTextChild(format("Here you can change %s settings.", DLU.addonNameColored), settingsPanel, settingsPanelTitle, 0, -8, "GameFontHighlightSmall");
	-- Setting buttons
	local checkX = -60;
	for t, s in pairs(DLUSettings) do
		local settingType = type(s);
		if (settingType == "table") then
			local checkBox = createCheckbutton(settingsPanel, 10, checkX, s.text, t);
			checkBox:SetChecked(s.value);
			checkBox:SetScript("OnClick", function()
				local settingText;
				
				if (checkBox:GetChecked()) then
					DLUSettings[t].value = true;
					settingText = DLU.colorizeString("enabled", "green");
				else
					DLUSettings[t].value = false;
					settingText = DLU.colorizeString("disabled", "red");
				end
				
				DLU.createMessage(format("%s option %s!", t, settingText));
			end);
			
			checkX = checkX - 20;
		end
	end
	
	local items = {
		"Blizzard",
		"DarkruneDK"
	};
	
	local dropDown = CreateFrame("Frame", "DLU_NumberFormatSetting", settingsPanel, "UIDropDownMenuTemplate")
	dropDown:SetPoint("TOPLEFT", 0, checkX - 20);
	UIDropDownMenu_SetWidth(dropDown, 200)
	UIDropDownMenu_JustifyText(dropDown, "LEFT")
	UIDropDownMenu_SetText(dropDown, DLUSettings["numberFormat"] .. " format");

	-- Create and bind the initialization function to the dropdown menu
	UIDropDownMenu_Initialize(dropDown, function(self, level, menuList)
		local info = UIDropDownMenu_CreateInfo()
		for _, text in pairs(items) do
			info.text, info.arg1 = text, text;
			if (DLUSettings["numberFormat"] == text) then
				info.checked = true;
			else
				info.checked = false;
			end
		
			info.menuList = 0;
			info.func = self.SetValue;
			UIDropDownMenu_AddButton(info);
		end
	end)
	
	function dropDown:SetValue(newValue)
		DLUSettings["numberFormat"] = newValue;
		-- Update the text; if we merely wanted it to display newValue, we would not need to do this
		UIDropDownMenu_SetText(dropDown, DLUSettings["numberFormat"] .. " format");
		-- Because this is called from a sub-menu, only that menu level is closed by default.
		CloseDropDownMenus()
	end
	
	-- Add reset button
	local resetButton = createButton(settingsPanel, "Reset", 100, 22, "Resets the options for " .. DLU.addonNameColored .. "." , 10, 30);
	resetButton:SetScript("OnClick", function()
		-- Reset Options
		DLU.resetAddonOptions();
		
		-- Update UI
		dropDown:SetValue(DLUSettings.numberFormat);
		for k,v in pairs(checkButtons) do
			v:SetChecked(DLUSettings[k]);
		end
	end);
	
	-- Artifact panel
	local artifactPanel = CreateFrame("Frame", "dluArtifactOptionPanel", UIParent);
	artifactPanel.name = "Artifact";
	artifactPanel.parent = panel.name;
	InterfaceOptions_AddCategory(artifactPanel);
	-- Artifact panel text
	local artiTitle = DLU.createTextRoot(artifactPanel.name, artifactPanel, 16, -16, "GameFontNormalLarge");
	local artiDescription = DLU.createTextChild("This page will have some buttons to help you track artifact achievements.", artifactPanel, artiTitle, 0, -8, "GameFontHighlightSmall");
	-- Hidden artifact	
	local hiddenArtifactAchievementTable = {
		["Dungeons completed"] = 11152,
		["World Quests completed"] = 11153,
		["Honorable kills"] = 11154
	};
	
	local buttonX = 10;
	for t, q in pairs(hiddenArtifactAchievementTable) do
		local trackText = "Track " .. t;
		local untrackText = "Untrack " .. t;
		local button = createButton(artifactPanel, trackText, 200, 22, "Tracks " .. t .. " needed for hidden artifact weapon appearance.", buttonX, 30);
		if (achievementCompleted(q)) then
			disableButton(button, "Already completed!");
		elseif (IsTrackedAchievement(q)) then
			button:SetText(untrackText);
		end
		button:SetScript("OnClick", function()
			local isTracked = IsTrackedAchievement(q);
			if (isTracked) then
				button:SetText(trackText);
				removeFromTracker(q);
			else
				button:SetText(untrackText);
				addToTracker(q);
			end
		end);
		buttonX = buttonX + 200;
	end
	
	-- Class specific trackers --
	local palaPanel = CreateFrame("Frame", "DLUPalaPanel", UIParent);
	palaPanel.name = DLU.getClassColored("Paladin", "PALADIN");
	palaPanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(palaPanel);
	-- Text
	local palaTitle = DLU.createTextRoot(palaPanel.name, palaPanel, 16, -16, "GameFontNormalLarge");
	local retArtiName = DLU.itemColorString(6, "Heart of Corruption");
	local palaText = DLU.createTextChild(format("%s requires Artifact Knowledge level 6 (Retribution).", retArtiName), palaPanel, palaTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local retButton = createButton(palaPanel, format("Check %s status", retArtiName), 220, 22, "Check how far you are to unlock the Corrupted Ashbringer.", 390, 30);
	retButton:SetScript("OnClick", function()
		DLU.createMessage("Not the exact order!", "error"); -- other color (not really an error)
		
		local steps = {
			["Has talked to Prince Tortheldrin"] = 43682;
		};
		
		if (DLU.player.faction == "Horde") then
			steps["Has talked to Bardu"] = 43683;
		else
			steps["Has talked to Alexia"] = 43683;
		end
		
		steps["Can slime drop Timolain"] = 43684;
		steps["Can Shard be fished up"] = 43685;
		
		DLU.tableStatus(steps);
	end);
	
	-- Mage --
	local magePanel = CreateFrame("Frame", "DLUMagePanel", UIParent);
	magePanel.name = DLU.getClassColored("Mage", "MAGE");
	magePanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(magePanel);
	-- Text
	local mageTitle = DLU.createTextRoot(magePanel.name, magePanel, 16, -16, "GameFontNormalLarge");
	local arcArtiName = DLU.itemColorString(6, "The Woolomancer's Charge");
	local mageText = DLU.createTextChild(format("%s required level 6 Artifact Knowledge (Arcane).", arcArtiName), magePanel, mageTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local arcButton = createButton(magePanel, format("Check %s status", arcArtiName), 260, 22, "Check how far you are to unlock the Woolomancer's Charge.", 350, 30);
	arcButton:SetScript("OnClick", function()
		local steps = {
			["Aszuna"] = 43787,
			["Stormheim"] = 43789,
			["Val'Sharah"] = 43790,
			["Suramar"] = 43791,
			["High Mountain"] = 43788
		}
		
		DLU.tableStatus(steps);
	end);
	
	-- Priest --
	local priestPanel = CreateFrame("Frame", "DLUPriestPanel", UIParent);
	priestPanel.name = DLU.getClassColored("Priest", "PRIEST");
	priestPanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(priestPanel);
	-- Text
	local priestTitle = DLU.createTextRoot(priestPanel.name, priestPanel, 16, -16, "GameFontNormalLarge");
	local discArtiName = DLU.itemColorString(6, "Writings of the End");
	local priestText = DLU.createTextChild(format("%s required Artifact Knowledge 4 (Discipline).", discArtiName), priestPanel, priestTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local discButton = createButton(priestPanel, format("Check %s Status", discArtiName), 220, 22, "Check how far you are to unlock the Writtings of the End.", 390, 30);
	discButton:SetScript("OnClick", function()
		DLU.createMessage("Not the exact order!", "error"); -- other color (not an error)
		
		local steps = {
			["Dalaran - The Violet Citadel"] = 44339,
			["Class Order Hall - Juvess the Duskwhisperer"] = 44340,
			["Northrend - New Hearthglen"] = 44341,
			["Class Order Hall - Archivist Inkforge "] = 44342,
			["Scholomance - Chillheart's Room"] = 44343,
			["Class Order Hall - Meridelle Lightspark"] = 44344,
			["Scarlet Halls - The Flameweaver's library"] = 44345,
			["Azsuna - Chief Bitterbrine Azsuna"] = 44346,
			["Suramar - Artificer Lothaire"] = 44347,
			["Black Rook Hold - Library after First Boss"] = 44348,
			["Karazhan - Guardian's Library"] = 44349,
			["Stormheim - Inquisitor Ernstonbok"] = 44350
		}
		
		DLU.tableStatus(steps);
	end);
	
	-- Warrior --
	local warPanel = CreateFrame("Frame", "DLUWarriorPanel", UIParent);
	warPanel.name = DLU.getClassColored("Warrior", "WARRIOR");
	warPanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(warPanel);
	-- Text
	local warTitle = DLU.createTextRoot(warPanel.name, warPanel, 16, -16, "GameFontNormalLarge");	
	local protArtiName = DLU.itemColorString(6, "Burning Plate of the Worldbreaker");
	local warText = DLU.createTextChild(format("%s required Artifact Knowledge 5 (Protection).", protArtiName), warPanel, warTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local protButton = createButton(warPanel, format("Check %s Status", protArtiName), 300, 22, "Check how far you are to unlock the Burning Plate of the Worldbreaker.", 310, 30);
	protButton:SetScript("OnClick", function()
		DLU.createMessage("Has to be available and not denied!", "error"); -- other color (not an error)
		
		local steps = {
			["Available"] = 44311,
			["Denied"] = 44312
		}
		
		DLU.tableStatus(steps);
	end);
end

-- WQCompanion
local BD = {
	bg = "Interface\\DialogFrame\\UI-DialogBox-Background",
	edge = "Interface\\Tooltips\\UI-Tooltip-Border"
}

local WIDTH = 520;
local HEIGHT_ERW = 200;
WQCompanion = CreateFrame("Frame", "DLU_WQCompanion", UIParent);

WQCompanion:SetBackdrop({bgFile=BD.bg, edgeFile=BD.edge, tile=false, edgeSize=12, insets={left=2.5, right=2.5, top=2.5, bottom=2.5}});
WQCompanion:SetWidth(WIDTH);
WQCompanion:SetHeight(600);
WQCompanion:SetPoint("CENTER", UIParent);
WQCompanion:SetFrameStrata("DIALOG");

WQCompanion.close = CreateFrame("Button", nil, WQCompanion, "UIPanelCloseButton")
WQCompanion.close:ClearAllPoints();
WQCompanion.close:SetPoint("TOPRIGHT", 0, 0);

WQCompanion.title = WQCompanion:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
WQCompanion.title:SetPoint("TOP", 0, -6);
WQCompanion.title:SetText("WQ Companion");

WQCompanion.EmissariesContainer = CreateFrame("Frame", nil, WQCompanion);
WQCompanion.EmissariesContainer:SetSize(WIDTH, 42);
WQCompanion.EmissariesContainer:SetPoint("TOPLEFT", WQCompanion, "TOPLEFT", 0, -20);

WQCompanion.EmissariesContainer.Emissaries = {};

for i = 1,3 do
	local labelText = "";
	local valueText = "Loading...";
	local tooltip = "Loading...";
	local questId = 0;
	
	if (DLU.emissaryTable[i]) then	
		local currentEmissary = DLU.emissaryTable[i];
		
		labelText = format("%s", DLU.iconToText(currentEmissary.icon));
		valueText = getEmissaryValue(i);
		tooltip = format("%s", currentEmissary.factionName);
		questId = currentEmissary.questId;
	end
	
	local columnWidth = WIDTH/3;
	
	WQCompanion.EmissariesContainer.Emissaries[i] = CreateFrame("Frame", nil, WQCompanion.EmissariesContainer);
	WQCompanion.EmissariesContainer.Emissaries[i]:SetWidth(columnWidth);
	WQCompanion.EmissariesContainer.Emissaries[i]:SetHeight(42);
	
	WQCompanion.EmissariesContainer.Emissaries[i].questId = questId;
	
	WQCompanion.EmissariesContainer.Emissaries[i]:SetPoint("TOPLEFT", WQCompanion.EmissariesContainer, "TOPLEFT", columnWidth*(i-1), 0);
	
	WQCompanion.EmissariesContainer.Emissaries[i].Label = WQCompanion.EmissariesContainer.Emissaries[i]:CreateFontString(nil, "ARTWORK", "GameFontWhite");
	WQCompanion.EmissariesContainer.Emissaries[i].Label:SetPoint("LEFT", 8, 0);
	WQCompanion.EmissariesContainer.Emissaries[i].Label:SetText(labelText);
	
	WQCompanion.EmissariesContainer.Emissaries[i].Value = WQCompanion.EmissariesContainer.Emissaries[i]:CreateFontString(nil, "ARTWORK", "GameFontWhite");
	WQCompanion.EmissariesContainer.Emissaries[i].Value:SetPoint("RIGHT", -8, 0);
	WQCompanion.EmissariesContainer.Emissaries[i].Value:SetText(valueText);
	
	WQCompanion.EmissariesContainer.Emissaries[i].Tooltip = tooltip;
	
	WQCompanion.EmissariesContainer.Emissaries[i]:SetScript("OnEnter", OnEnter);
	WQCompanion.EmissariesContainer.Emissaries[i]:SetScript("OnLeave", OnLeave);
end

WQCompanion.ERW = CreateFrame("ScrollFrame", nil, WQCompanion);
WQCompanion.ERW.Content = CreateFrame("Frame", nil, WQCompanion.ERW);
WQCompanion.ERW:SetScrollChild(WQCompanion.ERW.Content);
WQCompanion.ERW:SetScript("OnMouseWheel", function(self, sv)
	sv = sv * 16;
	local maximum = max(0, WQCompanion.ERW.Content:GetHeight() - self:GetHeight());
	local value = self:GetVerticalScroll();
	if ((value - sv) < 0) then
		self:SetVerticalScroll(0);
	elseif (value - sv) > maximum then
		self:SetVerticalScroll(maximum)
	else
		self:SetVerticalScroll(value - sv)
	end
end);
WQCompanion.ERW.Content:SetSize(WIDTH, HEIGHT_ERW);

WQCompanion.ERW.title = WQCompanion.ERW:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
WQCompanion.ERW.title:SetPoint("TOP", WQCompanion.ERW, 0, 15);
WQCompanion.ERW.title:SetText("Bonus rewards");

WQCompanion.ERW:SetSize(WIDTH, HEIGHT_ERW);
WQCompanion.ERW:SetPoint("TOPLEFT", WQCompanion.EmissariesContainer, 0, -50);
WQCompanion.ERW:SetBackdrop({bgFile=BD.bg, edgeFile=BD.edge, tile=false, edgeSize=12, insets={left=2.5, right=2.5, top=2.5, bottom=2.5}});

WQCompanion.ERW.Content.lines = {};
for i = 1, GetNumRFDungeons() do
	local line = CreateFrame("Frame", nil, WQCompanion.ERW.Content);
	line:SetPoint("TOPLEFT", 0, -(i-1)*16);
	line:SetSize(1, 16);
	
	line.text = line:CreateFontString(nil, "ARTWORK", "GameFontWhite");
	line.text:SetPoint("LEFT", 5, -5);
	
	WQCompanion.ERW.Content.lines[i] = line;
end

--[[
WQCompanion.WQL = CreateFrame("ScrollFrame", nil, WQCompanion);

WQCompanion.WQL:SetSize(WIDTH, HEIGHT_ERW);
WQCompanion.WQL:SetPoint("TOPLEFT", WQCompanion.ERW.Content, 0, -HEIGHT_ERW-15);
WQCompanion.WQL:SetBackdrop({bgFile=BD.bg, edgeFile=BD.edge, tile=false, edgeSize=12, insets={left=2.5, right=2.5, top=2.5, bottom=2.5}});

WQCompanion.WQL.title = WQCompanion.WQL:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
WQCompanion.WQL.title:SetPoint("TOP", WQCompanion.WQL, 0, 15);
WQCompanion.WQL.title:SetText("World Quest List");

WQCompanion.WQL.Content = CreateFrame("Frame", nil, WQCompanion.WQL);
WQCompanion.WQL:SetScrollChild(WQCompanion.WQL.Content);
WQCompanion.WQL:SetScript("OnMouseWheel", function(self, sv)
	sv = sv * 16;
	local maximum = max(0, WQCompanion.ERW.Content:GetHeight() - self:GetHeight());
	local value = self:GetVerticalScroll();
	if ((value - sv) < 0) then
		self:SetVerticalScroll(0);
	elseif (value - sv) > maximum then
		self:SetVerticalScroll(maximum)
	else
		self:SetVerticalScroll(value - sv)
	end
end);

WQCompanion.WQL.Content.lines = {};
for i = 1, 100 do
	local line = CreateFrame("Frame", nil, WQCompanion.WQL.Content);
	line:SetPoint("TOPLEFT", 0, -(i-1)*16);
	line:SetSize(1, 16);
	
	line.text = line:CreateFontString(nil, "ARTWORK", "GameFontWhite");
	line.text:SetPoint("LEFT", 5, -5);
	
	WQCompanion.WQL.Content.lines[i] = line;
end
]]

local function checkWQCompanionVisibility()
	if (DLU.player.level < 10 or not DLUSettings.showWQCompanionOnLogin.value) then
		WQCompanion:Hide();
	end
end

-- Minimap icon
local function iconToolTip(icon)
	icon:AddLine(DLU.addonNameColored);
	icon:AddLine("Open " .. DLU.addonName .. " help page.");
	icon:AddLine("May require 2 clicks once every session.");
end

local function iconClickEvent()
	if not InterfaceOptionsFrame:IsShown() then
        InterfaceOptionsFrame_OpenToCategory(DLU.addonName);
    else
        InterfaceOptionsFrame:Hide();
    end
end

local addon = LibStub("AceAddon-3.0"):NewAddon("DLU", "AceConsole-3.0")
local DLU_LDB = LibStub("LibDataBroker-1.1"):NewDataObject("DLU", {
	type = "Launcher",
	text = "DLU_Icon",
	icon = "Interface\\AddOns\\DarkruneDKLevelUp\\Images\\dlu_icon",
	OnClick = function() iconClickEvent() end,
	OnTooltipShow = function(tooltip) iconToolTip(tooltip) end
})
local icon = LibStub("LibDBIcon-1.0");

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("DLU_UI_DB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("DLU_Icon", DLU_LDB, self.db.profile.minimap)
end

-- Initialization
settingsFrame:RegisterEvent("PLAYER_LOGIN");
settingsFrame:SetScript("OnEvent", function() 

SetUpAddonOptions();
checkWQCompanionVisibility()
DLU.updateEmissaries();

settingsFrame:UnregisterEvent("PLAYER_LOGIN")
end);